(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit PaintUtils;

interface

uses
  Windows, Graphics;

(* Fonction de calcule de hauteur et de largeur *)

function WidthOf(R: TRect): Integer;
function HeightOf(R: TRect): Integer;

(* Fonction de calcule sur le texte *)

function LockSizeText(S: string; Size: longInt; C: Char) : string;
function MinimizeFileName (const Chaine : string; MaxSize : integer) : string;

(* Remplissage d'un Canvas par un dgrad *)

type
  TFillDirection = (fdTopToBottom, fdLeftToRight);

procedure GradientFillRect(Canvas: TCanvas; ARect: TRect; StartColor,
  EndColor: TColor; Direction: TFillDirection; Colors: Byte);

implementation

uses Math, SysUtils;

function WidthOf(R: TRect): Integer;
begin
  Result := R.Right - R.Left;
end;

function HeightOf(R: TRect): Integer;
begin
  Result := R.Bottom - R.Top;
end;

function LockSizeText(S: string; Size: Integer; C: Char) : string;
var Mask : string;
    I    : integer;
begin
  Mask := '';
  for I := 1 to Size
    do Mask := Mask + C;
  result := Copy(Mask, 1, Size - length(S)) + S;
end;

function MinimizeFileName (const Chaine : string; MaxSize : Integer) : string;
var PosS, PosR, FirstI : Integer;
    Del                : boolean;
begin
  result := Chaine;
  if Length (Chaine) < MaxSize then Exit;
  PosS   := 1;
  While Chaine [PosS] <> '\' do Inc (PosS);
  FirstI := PosS;
  Del    := True;
  PosR   := PosS + 1;
  While Del
  do begin
       Inc (PosS);
       If ((Chaine [PosS] = '\') and (Length (result) <= MaxSize)) or (LastDelimiter('\',result) = FirstI)
         then Del := false
         else Delete (Result, PosR, 1);
     end;
  Insert ('...', Result, PosR);
end;

procedure GradientFillRect(Canvas: TCanvas; ARect: TRect; StartColor,
  EndColor: TColor; Direction: TFillDirection; Colors: Byte);
var
  StartRGB: array[0..2] of Byte;    { Start RGB values }
  RGBDelta: array[0..2] of Integer; { Difference between start and end RGB values }
  ColorBand: TRect;                 { Color band rectangular coordinates }
  I, Delta: Integer;
  Brush: HBrush;
begin
  if IsRectEmpty(ARect) then Exit;
  if Colors < 2 then
  begin
    Brush := CreateSolidBrush (ColorToRGB (StartColor));
    FillRect (Canvas.Handle, ARect, Brush);
    DeleteObject (Brush);
    Exit;
  end;
  StartColor := ColorToRGB (StartColor);
  EndColor   := ColorToRGB (EndColor);
  { Set the Red, Green and Blue colors }
  StartRGB[0] := GetRValue(StartColor);
  StartRGB[1] := GetGValue(StartColor);
  StartRGB[2] := GetBValue(StartColor);
  { Calculate the difference between begin and end RGB values }
  RGBDelta[0] := GetRValue(EndColor) - StartRGB[0];
  RGBDelta[1] := GetGValue(EndColor) - StartRGB[1];
  RGBDelta[2] := GetBValue(EndColor) - StartRGB[2];
  { Calculate the color band's coordinates }
  ColorBand := ARect;
  if Direction = fdTopToBottom then
  begin
    Colors := Max (2, Min (Colors, HeightOf(ARect)));
    Delta  := HeightOf (ARect) div Colors;
  end
  else
  begin
    Colors := Max (2, Min (Colors, WidthOf(ARect)));
    Delta  := WidthOf (ARect) div Colors;
  end;
  with Canvas.Pen do
  begin { Set the pen style and mode }
    Style := psSolid;
    Mode := pmCopy;
  end;
  { Perform the fill }
  if Delta > 0 then
  begin
    for I := 0 to Colors do
    begin
      case Direction of
        { Calculate the color band's top and bottom coordinates }
        fdTopToBottom :
        begin
          ColorBand.Top := ARect.Top + I * Delta;
          ColorBand.Bottom := ColorBand.Top + Delta;
        end;
        { Calculate the color band's left and right coordinates }
        fdLeftToRight :
        begin
          ColorBand.Left := ARect.Left + I * Delta;
          ColorBand.Right := ColorBand.Left + Delta;
        end;
      end; {case}
      { Calculate the color band's color }
      Brush := CreateSolidBrush(RGB(
        StartRGB[0] + MulDiv(I, RGBDelta[0], Colors - 1),
        StartRGB[1] + MulDiv(I, RGBDelta[1], Colors - 1),
        StartRGB[2] + MulDiv(I, RGBDelta[2], Colors - 1)));
      FillRect(Canvas.Handle, ColorBand, Brush);
      DeleteObject(Brush);
    end;
  end;
  if Direction = fdTopToBottom
    then Delta := HeightOf (ARect) mod Colors
    else Delta := WidthOf  (ARect) mod Colors;
  if Delta > 0 then
  begin
    case Direction of
      { Calculate the color band's top and bottom coordinates }
      fdTopToBottom :
      begin
        ColorBand.Top    := ARect.Bottom - Delta;
        ColorBand.Bottom := ColorBand.Top + Delta;
      end;
      { Calculate the color band's left and right coordinates }
      fdLeftToRight :
      begin
        ColorBand.Left  := ARect.Right - Delta;
        ColorBand.Right := ColorBand.Left + Delta;
      end;
    end; {case}
    Brush := CreateSolidBrush(EndColor);
    FillRect(Canvas.Handle, ColorBand, Brush);
    DeleteObject(Brush);
  end;
end;

end.
